# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from djangojs.management.commands.subparser import Subparser


class LauncherParser(Subparser):
    name = 'launcher'
    help = 'Get a PhantomJS launcher path'

    def add_arguments(self, parser):
        parser.add_argument('name', help='Runner name')

    def handle(self, args):
        from djangojs.runners import LAUNCHERS
        launcher = args.name.lower()
        try:
            self.stdout.write(LAUNCHERS[launcher])
        except KeyError:
            self.stderr.write('Unknown launcher "%s"' % launcher)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class Subparser(object):
    '''
    A base class for js subcommands
    '''
    name = None
    help = None

    def __init__(self, command, subparsers):
        self.command = command
        subparser = subparsers.add_parser(self.name, help=self.help, description=self.help)
        subparser.set_defaults(func=self.handle)
        self.add_arguments(subparser)

    def add_arguments(self, parser):
        raise NotImplemented

    def handle(self, args):
        raise NotImplemented

    @property
    def stdout(self):
        return self.command.stdout

    @property
    def stderr(self):
        return self.command.stderr



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import logging
import re
import sys
import types

from django.core.serializers.json import DjangoJSONEncoder
from django.urls import URLPattern, URLResolver, get_script_prefix
from django.utils import six

from djangojs.conf import settings

logger = logging.getLogger(__name__)


__all__ = (
    'urls_as_dict',
    'urls_as_json',
)

RE_KWARG = re.compile(r"(\(\?P\<(.*?)\>.*?\))")  # Pattern for recongnizing named parameters in urls
RE_ARG = re.compile(r"(\(.*?\))")  # Pattern for recognizing unnamed url parameters
RE_OPT = re.compile(r"(?:\w|/)(?:\?|\*)")  # Pattern for recognizing optionnal character
RE_OPT_GRP = re.compile(r"\(\?\:.*\)(?:\?|\*)")  # Pattern for recognizing optionnal group
RE_ESCAPE = re.compile(r'([^\\]?)\\')  # Recognize escape characters
RE_START_END = re.compile(r'[\$\^]')  # Recognize start and end charaters

try:  # check for django-cms
    from cms.appresolver import AppRegexURLResolver
    CMS_APP_RESOLVER = True
except:
    CMS_APP_RESOLVER = False  # we can live without it


def urls_as_dict():
    '''
    Get the URLs mapping as a dictionnary
    '''
    module = settings.JS_URL_CONFS
    urls = {}
    if settings.JS_URLS_ENABLED:
        if isinstance(module, (six.text_type, six.string_types)):
            urls = _get_urls(module)
        else:
            for item in module:
                urls = dict(urls.items() + _get_urls(item).items())
    else:
        return {}

    return urls


def urls_as_json():
    '''
    Get the URLs mapping as JSON
    '''
    return json.dumps(urls_as_dict(), cls=DjangoJSONEncoder)


def _get_urls_for_pattern(pattern, prefix='', namespace=None):
    urls = {}

    if prefix is '':
        prefix = get_script_prefix()

    if issubclass(pattern.__class__, URLPattern):
        if settings.JS_URLS_UNNAMED:
            mod_name, obj_name = pattern.callback.__module__, pattern.callback.__name__
            try:
                module = __import__(mod_name, fromlist=[obj_name])
                obj = getattr(module, obj_name)
                func_name = "{0}.{1}".format(mod_name, obj_name) if isinstance(obj, types.FunctionType) else None
                pattern_name = pattern.name or func_name
            except:
                pattern_name = pattern.name
        else:
            pattern_name = pattern.name

        if pattern_name:
            if settings.JS_URLS and pattern_name not in settings.JS_URLS:
                return {}
            if settings.JS_URLS_EXCLUDE and pattern_name in settings.JS_URLS_EXCLUDE:
                return {}
            if namespace:
                pattern_name = ':'.join((namespace, pattern_name))
            full_url = prefix + pattern.pattern.regex.pattern
            for char in ['^', '$']:
                full_url = full_url.replace(char, '')
            # remove optionnal non capturing groups
            opt_grp_matches = RE_OPT_GRP.findall(full_url)
            if opt_grp_matches:
                for match in opt_grp_matches:
                    full_url = full_url.replace(match, '')
            # remove optionnal characters
            opt_matches = RE_OPT.findall(full_url)
            if opt_matches:
                for match in opt_matches:
                    full_url = full_url.replace(match, '')
            # handle kwargs, args
            kwarg_matches = RE_KWARG.findall(full_url)
            if kwarg_matches:
                for el in kwarg_matches:
                    # prepare the output for JS resolver
                    full_url = full_url.replace(el[0], "<%s>" % el[1])
            # after processing all kwargs try args
            args_matches = RE_ARG.findall(full_url)
            if args_matches:
                for el in args_matches:
                    full_url = full_url.replace(el, "<>")  # replace by a empty parameter name
            # Unescape charaters
            full_url = RE_ESCAPE.sub(r'\1', full_url)
            urls[pattern_name] = full_url
    elif (CMS_APP_RESOLVER) and (issubclass(pattern.__class__, AppRegexURLResolver)):  # hack for django-cms
        for p in pattern.url_patterns:
            urls.update(_get_urls_for_pattern(p, prefix=prefix, namespace=namespace))
    elif issubclass(pattern.__class__, URLResolver):
        if pattern.urlconf_name:
            if pattern.namespace and not pattern.app_name:
                # Namespace without app_name
                nss = [pattern.namespace]
            else:
                # Add urls twice: for app and instance namespace
                nss = set((pattern.namespace, pattern.app_name))
            for ns in nss:
                namespaces = [nsp for nsp in (namespace, ns) if nsp]
                namespaces = ':'.join(namespaces)
                if settings.JS_URLS_NAMESPACES and namespaces and namespaces not in settings.JS_URLS_NAMESPACES:
                    continue
                if settings.JS_URLS_NAMESPACES_EXCLUDE and namespaces in settings.JS_URLS_NAMESPACES_EXCLUDE:
                    continue
                new_prefix = '%s%s' % (prefix, pattern.pattern.regex.pattern)
                urls.update(_get_urls(pattern.urlconf_name, new_prefix, namespaces))

    return urls


def _get_urls(module, prefix='', namespace=None):
    urls = {}
    if isinstance(module, (six.text_type, six.string_types)):
        try:
            __import__(module)
            root_urls = sys.modules[module]
            patterns = root_urls.urlpatterns
        except ImportError:  # die silently
            patterns = tuple()
    elif isinstance(module, (list, tuple)):
        patterns = module
    elif isinstance(module, types.ModuleType):
        patterns = module.urlpatterns
    else:
        raise TypeError('Unsupported type: %s' % type(module))

    for pattern in patterns:
        urls.update(_get_urls_for_pattern(pattern, prefix=prefix, namespace=namespace))

    return urls



# -*- coding: utf-8 -*-
from django import VERSION as DJANGO_VERSION
from django.db import models


class FakeModel(models.Model):
    something = models.CharField(max_length=256)

    class Meta:
        permissions = (
            ("do_something", "Can do something"),
            ("do_something_else", "Can do something else"),
        )


if DJANGO_VERSION >= (1, 5):
    from django.contrib.auth.models import AbstractBaseUser, UserManager

    class CustomUser(AbstractBaseUser):
        identifier = models.CharField(max_length=40, unique=True, db_index=True)
        USERNAME_FIELD = 'identifier'

        objects = UserManager()



# -*- coding: utf-8 -*-
from django.utils import six
from djangojs.context_serializer import ContextSerializer


class SocialAuthContextMixin(object):
    '''Handle django_social_auth context specifics'''
    def process_social_auth(self, social_auth, data):
        """ Just force social_auth's LazyDict to be converted to a dict for the
            JSON serialization to work properly. """

        data['social_auth'] = dict(six.iteritems(social_auth))


class SocialAuthContextSerializer(SocialAuthContextMixin, ContextSerializer):
    '''Already packed django_social_auth ContextSerializer'''
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import re

from glob import iglob

from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import CommandError
from djangojs.management.commands.subparser import Subparser


class LocalizeParser(Subparser):
    '''
    A command handling localization.
    '''
    name = 'localize'
    help = 'Generate PO file from js files'

    def add_arguments(self, parser):
        parser.add_argument('apps', nargs='*', metavar='app', help="Applications to localize")
        parser.add_argument('--locale', '-l',
            help='Creates or updates the message files for the given locale (e.g. pt_BR).'),
        parser.add_argument('--all', '-a', action='store_true',
            help='Updates the message files for all existing locales.'),
        parser.add_argument('--extension', '-e', dest='extensions', default=['js'], action='append',
            help=('The file extension(s) to examine (default: "js"). '
            'Separate multiple extensions with commas, or use -e multiple times.')),
        parser.add_argument('--symlinks', '-s', action='store_true',
            help='Follows symlinks to directories when examining source code and templates for translation strings.'),
        parser.add_argument('--ignore', '-i', action='append', default=[], metavar='PATTERN',
            help='Ignore files or directories matching this glob-style pattern. Use multiple times to ignore more.'),
        parser.add_argument('--no-default-ignore', action='store_false',
            help="Don't ignore the common glob-style patterns 'CVS', '.*' and '*~'."),
        parser.add_argument('--no-wrap', action='store_true',
            help="Don't break long message lines into several lines"),
        parser.add_argument('--no-location', action='store_true',
            help="Don't write '#: filename:line' lines"),
        parser.add_argument('--no-obsolete', action='store_true', help="Remove obsolete message strings"),

    def handle(self, args):
        from django.core.management.commands.makemessages import make_messages, handle_extensions
        from django.db import models
        from djangojs.conf import settings

        if not args.apps:
            raise CommandError('Enter at least one application name.')

        if not (args.locale or args.all):
            raise CommandError('Enter a locale or use the --all switch.')

        ignore_patterns = args.ignore
        if args.no_default_ignore:
            ignore_patterns += ['CVS', '.*', '*~']
        ignore_patterns = list(set(ignore_patterns))

        extensions = handle_extensions(args.extensions)

        try:
            apps = [models.get_app(app) for app in args.apps]
        except (ImproperlyConfigured, ImportError) as e:
            raise CommandError("%s. Are you sure your INSTALLED_APPS setting is correct?" % e)

        for app in apps:
            app_dir = os.path.dirname(app.__file__)
            os.chdir(app_dir)

            catalogs = [self.build_catalog(root, ext, regexp) for ext, root, regexp in settings.JS_I18N_PATTERNS]

            make_messages(args.locale, b'djangojs', args.verbosity, args.all, extensions,
                          args.symlinks, ignore_patterns, args.no_wrap, args.no_location,
                          args.no_obsolete, self.stdout)

            for catalog in catalogs:
                os.remove(catalog)

    def build_catalog(self, root, extension, regexp):
        catalog = os.path.join(root, '{}_catalog.js'.format(extension))
        regexps = regexp if isinstance(regexp, (list, tuple)) else [regexp]
        with open(catalog, 'wb') as output:
            for _, dirnames, _ in os.walk(root):
                for dirname in [''] + dirnames:
                    glob_pattern = os.path.join(root, dirname, '*.{}'.format(extension))
                    for filename in iglob(glob_pattern):
                        content = open(filename, 'r').read()
                        for regexp in regexps:
                            for match in re.finditer(regexp, content):
                                line_no = content.count('\n', 0, match.start()) + 1
                                output.write('// Translators: {}:{}\n'.format(filename, line_no))
                                output.write('gettext(\'{}\');\n'.format(match.group(1)))
        return catalog



# -*- coding: utf-8 -*-
'''
This modules holds every helpers that does not fit in any standard django modules.
'''
from __future__ import unicode_literals

import logging
import os
import sys

from django.contrib.staticfiles import finders
from django.contrib.staticfiles.utils import matches_patterns
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.encoding import force_text
from django.utils.functional import Promise

logger = logging.getLogger(__name__)


__all__ = (
    'class_from_string',
    'LazyJsonEncoder',
    'StorageGlobber',
)


def class_from_string(name):
    '''
    Get a python class object from its name
    '''
    module_name, class_name = name.rsplit('.', 1)
    __import__(module_name)
    module = sys.modules[module_name]
    return getattr(module, class_name)


class LazyJsonEncoder(DjangoJSONEncoder):
    '''
    A JSON encoder handling promises (aka. Django lazy objects).

    See: https://docs.djangoproject.com/en/dev/topics/serialization/#id2
    '''
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return super(LazyJsonEncoder, self).default(obj)


class StorageGlobber(object):
    '''
    Retrieve file list from static file storages.
    '''
    @classmethod
    def glob(cls, files=None):
        '''
        Glob a pattern or a list of pattern static storage relative(s).
        '''
        files = files or []
        if isinstance(files, str):
            files = os.path.normpath(files)
            matches = lambda path: matches_patterns(path, [files])
            return [path for path in cls.get_static_files() if matches(path)]
        elif isinstance(files, (list, tuple)):
            all_files = cls.get_static_files()
            files = [os.path.normpath(f) for f in files]
            sorted_result = []
            for pattern in files:
                sorted_result.extend([f for f in all_files if matches_patterns(f, [pattern])])
            return sorted_result

    @classmethod
    def get_static_files(cls):
        files = []
        for finder in finders.get_finders():
            for path, storage in finder.list(None):
                # Prefix the relative path if the source storage contains it
                if getattr(storage, 'prefix', None):
                    prefixed_path = os.path.join(storage.prefix, path)
                else:
                    prefixed_path = path

                if prefixed_path not in files:
                    files.append(os.path.normpath(prefixed_path))
        return files



# -*- coding: utf-8 -*-
'''
Main access point for all JS commands
'''
import argparse
import logging
import sys

from django.core.management.base import BaseCommand, handle_default_options
try:
    from django.core.management.base import OutputWrapper
except:
    pass

from djangojs.management.commands.js_localize import LocalizeParser
from djangojs.management.commands.js_launcher import LauncherParser
from djangojs.management.commands.js_bower import BowerParser

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Handle javascript operations'
    args = 'command'
    requires_model_validation = False
    can_import_settings = True
    subparsers = (
        BowerParser,
        LauncherParser,
        LocalizeParser,
    )

    def usage(self, subcommand):
        return self.create_parser('', subcommand).format_usage()

    def run_from_argv(self, argv):
        """
        Set up any environment changes requested (e.g., Python path
        and Django settings), then run this command. If the
        command raises a ``CommandError``, intercept it and print it sensibly
        to stderr.
        """
        parser = self.create_parser(argv[0], argv[1])
        args = parser.parse_args(argv[2:])
        handle_default_options(args)
        try:
            self.execute(args)
        except Exception as e:
            # self.stderr is not guaranteed to be set here
            try:
                fallback_stderr = OutputWrapper(sys.stderr, self.style.ERROR)
            except:
                fallback_stderr = self.stdout
            stderr = getattr(self, 'stderr', fallback_stderr)
            if args.traceback:
                stderr.write(traceback.format_exc())
            else:
                stderr.write('%s: %s' % (e.__class__.__name__, e))
            sys.exit(1)

    def create_parser(self, prog_name, subcommand):
        """
        Create and return the ``OptionParser`` which will be used to
        parse the arguments to this command.

        """
        parser = argparse.ArgumentParser(prog='%s %s' % (prog_name, subcommand), description=self.help)

        parser.add_argument('-v', '--verbosity', action='store', default=1, type=int, choices=range(4),
            help='Verbosity level; 0=minimal output, 1=normal output, 2=verbose output, 3=very verbose output'),
        parser.add_argument('--settings',
            help='The Python path to a settings module, e.g. "myproject.settings.main". '
            'If this isn\'t provided, the DJANGO_SETTINGS_MODULE environment variable will be used.'),
        parser.add_argument('--pythonpath',
            help='A directory to add to the Python path, e.g. "/home/djangoprojects/myproject".'),
        parser.add_argument('--traceback', action='store_true', help='Print traceback on exception'),

        subparsers = parser.add_subparsers(description='JavaScript command to execute')

        for subparser in self.subparsers:
            subparser(self, subparsers)

        return parser

    def print_help(self, prog_name, subcommand):
        parser = self.create_parser(prog_name, subcommand)
        parser.print_help()

    def handle(self, args):
        args.func(args)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import logging

from django.conf import settings
from django.template.context import RequestContext
from django.utils import translation, six

from djangojs.conf import settings
from djangojs.utils import LazyJsonEncoder

logger = logging.getLogger(__name__)


__all__ = (
    'ContextSerializer',
)

SERIALIZABLE_TYPES = six.string_types + six.integer_types + (six.text_type, tuple, list, dict, bool, set)


class ContextSerializer(object):
    '''
    Serialize context and user from requests.

    Inherits from this class and set your :ref:`settings.JS_CONTEXT_PROCESSOR <js-context-processor>`
    to customize the serialization.

    To add a custom variable serialization handler,
    add a method named ``process_VARNAME(self, value, data)``.
    '''

    def __init__(self, request):
        self.request = request

    def as_dict(self):
        '''
        Serialize the context as a dictionnary from a given request.
        '''
        data = {}
        if settings.JS_CONTEXT_ENABLED:
            for context in RequestContext(self.request):
                for key, value in six.iteritems(context):
                    if settings.JS_CONTEXT and key not in settings.JS_CONTEXT:
                        continue
                    if settings.JS_CONTEXT_EXCLUDE and key in settings.JS_CONTEXT_EXCLUDE:
                        continue
                    handler_name = 'process_%s' % key
                    if hasattr(self, handler_name):
                        handler = getattr(self, handler_name)
                        data[key] = handler(value, data)
                    elif isinstance(value, SERIALIZABLE_TYPES):
                        data[key] = value
        if settings.JS_USER_ENABLED:
            self.handle_user(data)
        # Monkey patch for https://github.com/noirbizarre/django.js/issues/53
        data.update({
            'STATIC_URL': settings.STATIC_URL,
            'MEDIA_URL': settings.MEDIA_URL,
            'LANGUAGES': settings.LANGUAGES,
            'LANGUAGE_CODE': settings.LANGUAGE_CODE,
        })
        return data

    def as_json(self):
        '''
        Serialize the context as JSON.
        '''
        return json.dumps(self.as_dict(), cls=LazyJsonEncoder)

    def process_LANGUAGES(self, languages, data):
        '''Serialize LANGUAGES as a localized dictionnary.'''
        return dict(languages)

    def process_LANGUAGE_CODE(self, language_code, data):
        '''
        Fix language code when set to non included default `en`
        and add the extra variables ``LANGUAGE_NAME`` and ``LANGUAGE_NAME_LOCAL``.
        '''
        # Dirty hack to fix non included default
        language_code = 'en-us' if language_code == 'en' else language_code
        language = translation.get_language_info('en' if language_code == 'en-us' else language_code)
        if not settings.JS_CONTEXT or 'LANGUAGE_NAME' in settings.JS_CONTEXT \
            or (settings.JS_CONTEXT_EXCLUDE and 'LANGUAGE_NAME' in settings.JS_CONTEXT_EXCLUDE):
            data['LANGUAGE_NAME'] = language['name']
        if not settings.JS_CONTEXT or 'LANGUAGE_NAME_LOCAL' in settings.JS_CONTEXT \
            or (settings.JS_CONTEXT_EXCLUDE and 'LANGUAGE_NAME_LOCAL' in settings.JS_CONTEXT_EXCLUDE):
            data['LANGUAGE_NAME_LOCAL'] = language['name_local']
        return language_code

    def handle_user(self, data):
        '''
        Insert user informations in data

        Override it to add extra user attributes.
        '''
        # Default to unauthenticated anonymous user
        data['user'] = {
            'id': None,
            'username': '',
            'is_authenticated': False,
            'is_staff': False,
            'is_superuser': False,
            'permissions': tuple(),
        }
        if 'django.contrib.sessions.middleware.SessionMiddleware' in settings.MIDDLEWARE:
            user = self.request.user
            data['user']['is_authenticated'] = user.is_authenticated
            data['user']['id'] = user.id
            if hasattr(user, 'username'):
                data['user']['username'] = user.username
            elif hasattr(user, 'get_username'):
                data['user']['username'] = user.get_username()
            if hasattr(user, 'is_staff'):
                data['user']['is_staff'] = user.is_staff
            if hasattr(user, 'is_superuser'):
                data['user']['is_superuser'] = user.is_superuser
            if hasattr(user, 'get_all_permissions'):
                data['user']['permissions'] = tuple(user.get_all_permissions())



# -*- coding: utf-8 -*-
'''
This package is dedicated to contributed compatibility modules.

These modules provide mixins and already packed
:class:`ContextSerializers <djangojs.context_serializer.ContextSerializer>`
to use Django applications incompatible with the defaut Django.js behavior.
'''



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json

from os.path import exists

from django.core.management.base import CommandError
from djangojs.management.commands.subparser import Subparser


class BowerParser(Subparser):
    name = 'bower'
    help = 'Generate a .bowerrc file'

    def add_arguments(self, parser):
        parser.add_argument('target', help='The target directory for bower downloads')
        parser.add_argument('-f', '--force', action='store_true', help='Overwrite the file if exists')

    def handle(self, args):
        filename = '.bowerrc'
        if exists(filename) and not args.force:
            raise CommandError('.bowerrc file already exists. Use --force to overwrite')

        target = args.target if args.target.endswith('/') else '{}/'.format(args.target)
        bowerrc = {
            'directory': './{}'.format(target)
        }

        with open(filename, 'w') as output:
            output.write(json.dumps(bowerrc))
            output.write('\n')

        self.stdout.write('Created .bowerrc file into the current directory')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys
import codecs

from setuptools import setup, find_packages


if sys.argv[-1] == 'wheel':
    os.system('pip wheel --wheel-dir=wheelhouse .')
    sys.exit()

PYPI_RST_FILTERS = (
    # Replace code-blocks
    (r'\.\.\s? code-block::\s*(\w|\+)+',  '::'),
    # Remove travis ci badge
    (r'.*travis-ci\.org/.*', ''),
    # Remove pypip.in badges
    (r'.*pypip\.in/.*', ''),
    (r'.*crate\.io/.*', ''),
    (r'.*coveralls\.io/.*', ''),
)


def rst(filename):
    '''
    Load rst file and sanitize it for PyPI.
    Remove unsupported github tags:
     - code-block directive
     - travis ci build badge
    '''
    content = codecs.open(filename, encoding='utf-8').read()
    for regex, replacement in PYPI_RST_FILTERS:
        content = re.sub(regex, replacement, content)
    return content


long_description = '\n'.join((
    rst('README.rst'),
    rst('CHANGELOG.rst'),
    ''
))

install_requires = ['django']
if sys.version_info[0:2] < (2, 7):
    install_requires += ['argparse']

setup(
    name='django.js-ng',
    version=__import__('djangojs').__version__,
    description=__import__('djangojs').__description__,
    long_description=long_description,
    url='https://github.com/Robert-Chiang/django.js',
    download_url='http://pypi.python.org/pypi/django.js-ng',
    author='Robert Chiang',
    author_email='robert@streetvoice.com',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    license='LGPL',
    keywords='django javascript test url reverse helpers',
    classifiers=[
        "Framework :: Django",
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: System :: Software Distribution",
        "Programming Language :: Python",
        'Programming Language :: Python :: 3',
        "Topic :: Software Development :: Libraries :: Python Modules",
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
    ],
)
